/*
 * Device Driver for DataTranslation DT2801
 *
 * $Id: dt2801.h,v 1.2 1996/03/13 09:35:27 martin Exp martin $
 * $Source: /home/martin/Software/DAQ/drivers/dt2801/RCS/dt2801.h,v $
 *
 *
 * $Log: dt2801.h,v $
 * Revision 1.2  1996/03/13 09:35:27  martin
 * Cleanup, fake mode
 *
 * Revision 1.1  1996/03/04  07:18:55  martin
 * Initial revision
 *
 *
 */

#ifndef _DT2801_H
#define _DT2801_H  1

#ifdef __KERNEL__

#include "dt2801defs.h"

/* See Linux Journal 3/96 p.13 */
#ifdef DEBUG_DT2801
#define DT_PRINTK(format, args...)  printk (format, ## args)
#else
#define DT_PRINTK(format, args...)
#endif

/* Old version of printk-dbugging, will be removed when all statements have been
   converted to the new version
 */
#ifdef DEBUG_DT2801
#define dprintk( a )  printk a 
#else
#define dprintk( a )
#endif



/* Hardware Configuration */
/* ====================== */

/*
 *  General
 */

/* Major device number */
#ifndef DT2801_MAJOR
#define DT2801_MAJOR 19
#endif

/* I/O-port Baseaddress. Jumper selectable 10bit, last bit always zero
   (even address).
 */
#ifndef DT2801_BASEADDR
#define DT2801_BASEADDR   0x2ec
#endif

/* Channel for DMA-transfers */
#ifndef DT2801_DMACHANNEL
#define DT2801_DMACHANNEL    1
#endif

#define DT2801_MAX_DMA_SIZE (64 * 1024)

/* Ports */
#define DT2801_IOSIZE 2
#define DT_DATA_IN  (DT2801_BASEADDR)
#define DT_DATA_OUT (DT2801_BASEADDR)
#define DT_STATUS   (DT2801_BASEADDR+1)
#define DT_COMMAND  (DT2801_BASEADDR+1)


/* define's & typedef's */
/* ====================== */

/* Commands */
#define DT_C_RESET       0x0
#define DT_C_CLEAR_ERR   0x1
#define DT_C_READ_ERRREG 0x2
#define DT_C_SET_CLOCK   0x3

#define DT_C_TEST        0xb
#define DT_C_STOP        0xf

#define DT_C_SET_DIGIN   0x4
#define DT_C_SET_DIGOUT  0x5
#define DT_C_READ_DIG    0x6
#define DT_C_WRITE_DIG   0x7

#define DT_C_WRITE_DAIM  0x8
#define DT_C_SET_DA      0x9
#define DT_C_WRITE_DA    0xa

#define DT_C_READ_ADIM   0xc
#define DT_C_SET_AD      0xd
#define DT_C_READ_AD     0xe

/* Command modifiers (only used with read/write), EXTTRIG can be used 
   with some other commands.
*/
#define DT_MOD_DMA     (1<<4)
#define DT_MOD_CONT    (1<<5)
#define DT_MOD_EXTCLK  (1<<6)
#define DT_MOD_EXTTRIG (1<<7)

/* Bits in status register */
#define DT_S_DATA_OUT_READY   (1<<0)
#define DT_S_DATA_IN_FULL     (1<<1)
#define DT_S_READY            (1<<2)
#define DT_S_COMMAND          (1<<3)
#define DT_S_COMPOSITE_ERROR  (1<<7)

#define DT2801_TIMERCYCLES 1


#include "dt2801_adc.h"
/* #include "dt2801_dac.h" */
/* #include "dt2801_dio.h" */

/* Parameters for the whole board */

typedef struct {
    int ioregion;
    int dmachannel;
} dt2801Res_t;

typedef struct {

    int fake;

    int major;
    unsigned long baseaddr;
    int dmachannel;

    int inuse;

    int inerror;
    int stat;

    int dmamode;
    int continous;
    int exttrigger;
    int extclock;

/* The clock (ticks) */
    unsigned int clock_min;
    unsigned int clock_max;
    int clock;

  /* int num_adcs; */
    Dt2801_ADC_t *adcs;

/*
    int num_dacs;
    Dt2801_DAC_t *dacs;
*/

   /* Dt2801_DIO_t dio; */
} Dt2801_t;

extern Dt2801_t *dt2801;


extern int dt2801_readdata(int*);
extern int dt2801_readdata2(int*);
extern int dt2801_writedata(unsigned int);
extern int dt2801_writedata2(unsigned int);

extern int dt2801_writecommand(int);

extern int dt2801_ioctl(struct inode*, struct file*,
                         unsigned int, unsigned long);
extern void dt2801_release(struct inode*, struct file*);
extern int dt2801_open(struct inode*, struct file*);
extern int dt2801_stop(void);
extern int dt2801_clearerror(void);
extern int dt2801_reset(void);
extern int dt2801_checkforerror(void);


#endif /* __KERNEL__ */

/* ioctl's */
#define DT2801_IOCTL_STOP          0x01
#define DT2801_IOCTL_RESET         0x02
#define DT2801_IOCTL_SETCLOCK      0x03
#define DT2801_IOCTL_SETEXTCLOCK   0x04
#define DT2801_IOCTL_SETEXTTRIGGER 0x05
#define DT2801_IOCTL_SETDMA        0x06
#define DT2801_IOCTL_SETCONTINOUS  0x07

#endif /* _DT2801_H */
