/*
 * Device Driver for DataTranslation DT2801
 *
 * $Id: dt2801_adc.c,v 1.3 1996/04/09 16:03:04 martin Exp $
 * $Source: /home/martin/Software/DAQ/drivers/dt2801/RCS/dt2801_adc.c,v $
 *
 *
 * $Log: dt2801_adc.c,v $
 * Revision 1.3  1996/04/09 16:03:04  martin
 * no reset() at adc_open
 *
 * Revision 1.2  1996/03/13 09:36:07  martin
 * Fake mode, DMA-patches, cleanup
 *
 * Revision 1.1  1996/03/04 07:19:57  martin
 * Initial revision
 *
 *
 */

#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/tty.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/malloc.h>
#include <linux/timer.h>

#include <asm/io.h>
#include <asm/dma.h>
#include <asm/segment.h>
#include <asm/system.h>

#ifdef MODULE
#include <linux/version.h>
#else
#error "dt2801_adc must be compiled as a module"
#endif

#include "dt2801.h"
#include "dt2801_adc.h"

static struct timer_list DT2801_timer_list;
static int fake_counter = 0;

static void DT2801_start_timer(void);

static void DT2801_timer_interrupt(unsigned long unused)
{
    dprintk(( "DT2801 timer interrupt\n" ));
    del_timer(&DT2801_timer_list);
    if (dt2801->fake != 0) 
    {
        short *b = (short *) dt2801->adcs->dmabuf;

        b[fake_counter] = fake_counter%5;
        fake_counter++;
        DT2801_start_timer();
    }
}

static void DT2801_start_timer(void)
{

    if (fake_counter >= dt2801->adcs->conv) {
        return;
    }
    init_timer(&DT2801_timer_list);
    DT2801_timer_list.expires = jiffies + 20;
    DT2801_timer_list.function = DT2801_timer_interrupt;
    add_timer(&DT2801_timer_list);
}

/* end of fake routines */

static int allocate_adc_buffers(int bufsize)
{
    Dt2801_ADC_t* adc = dt2801->adcs;

    if ( bufsize == 0 || bufsize < 0 ) {
        printk( "dt2801_adc: allocating DMA-buffer, illegal bufsize %d\n", bufsize);
        return -(EINVAL);
    }
    if (bufsize > DT2801_MAX_DMA_SIZE) {
        printk( "dt2801_adc: Requested DMA buffer size, %d, too large\n",
              bufsize );
        return -(EINVAL);
    }

    /* reuse the buffer if it is large enough */
    if (bufsize <= adc->dmabufsize ) {
        return 0;
    }
 
    if (adc->dmabuf != NULL) 
    {
        printk( "dt2801_adc: freeing dmabuf\n");
        kfree(adc->dmabuf);
        adc->dmabufsize = 0;
    }


    printk( "dt2801_adc: allocating\n");

    /* Allocate buffers for DMA transfers */
    adc->dmabuf = kmalloc( bufsize, GFP_KERNEL | GFP_DMA );
    if (adc->dmabuf == NULL) {
            printk( "dt2801_adc: Unable to allocate DMA buffer\n" );
            return -(ENOMEM);
    }
    adc->dmabufsize = bufsize;

    return 0;
}

static int dt2801_adc_ioctl(struct inode *inode, struct file *file,
                       unsigned int cmd, unsigned long arg)
{
    Dt2801_ADC_t *adc = dt2801->adcs;

/*
    dprintk(( "dt2801_adc_ioctl: %d.\n", cmd ));
*/

    switch (cmd)
    {

        case DT2801_IOCTL_ADC_SET_PARAMS: {
                Dt2801_ADCP_t adcp;
                int stat;

                memcpy_fromfs(&adcp, (char *)arg, sizeof(adcp));
                dprintk(( "dt2801_adc_ioctl: gain = %d.\n", adcp.gain ));
                dprintk(( "dt2801_adc_ioctl: startch = %d.\n", adcp.startch ));
                dprintk(( "dt2801_adc_ioctl: endch = %d.\n", adcp.endch ));
                dprintk(( "dt2801_adc_ioctl: conv = %u.\n", adcp.conv ));
                dprintk(( "dt2801_adc_ioctl: conv = %u.\n", 
                                                      adcp.conv*sizeof(int) ));

                dt2801_writecommand(DT_C_SET_AD);

                dt2801_writedata(adcp.gain);
                dt2801_writedata(adcp.startch);
                dt2801_writedata(adcp.endch);
                dt2801_writedata2(adcp.conv);

                adc->last = 0;
                adc->gain = adcp.gain;
                adc->startch = adcp.startch;
                adc->endch = adcp.endch;
                adc->conv = adcp.conv;
                adc->cnt = 0;

                if (dt2801->dmamode == 0) {
                    return dt2801_checkforerror();

                } else {
                    adc->blocksize = adc->conv * sizeof(short);
                    if ( (stat = allocate_adc_buffers(adc->blocksize)) != 0) {
                         return stat;
                    }

                    return dt2801_checkforerror();
                }
            }
            break;
        case DT2801_IOCTL_ADC_SET_MODE: {
                dprintk(( "dt2801_adc:ioctl setmode\n" ));
                switch (arg)
                {
                case DT2801_ADC_IMMEDIATE:
                     adc->mode = arg;
                break;
                case DT2801_ADC_BLOCK:
                     if (dt2801->dmamode == 0) {
                         printk("dt2801_adc_ioctl: block only with dma\n");
                         return -1;
                     } else {
                         dprintk(( "dt2801_adc:ioctl block-dma\n" ));
                     }
                     adc->mode = arg;
                break;
                default:
                     printk("dt2801_adc_ioctl: wrong arg %lu to setmode\n", arg);
                     return -1;
                }
            }
            break;
        case DT2801_IOCTL_ADC_BLOCKSTART: {
                dprintk(( "dt2801_adc: block runit\n" ));
                if (dt2801->dmamode == 0) {
                    dprintk(( "dt2801_adc_ioctl: block only with dma\n" ));
                    return -1;
                } else  {
                    dprintk(( "dt2801: blockstart-dma\n" ));
                    dprintk(( "dt2801: cnt %d\n", adc->conv ));
                    if (dt2801->fake != 0) {
                      /* Start timer to fill the buffer */
                        fake_counter = 0;
                        DT2801_start_timer();
                        return 0;

                    } else {
                      /*    cli();    XXX */
                        disable_dma(dt2801->dmachannel);
                        clear_dma_ff( dt2801->dmachannel );
                        set_dma_addr( dt2801->dmachannel, 
                                      (unsigned long)adc->dmabuf );
                        set_dma_count( dt2801->dmachannel,
                                       adc->dmabufsize );
                        set_dma_mode( dt2801->dmachannel, DMA_MODE_READ );
                        enable_dma(dt2801->dmachannel);

                        if (dt2801->exttrigger == 1) {
                          dt2801_writecommand(
                                              DT_C_READ_AD | DT_MOD_DMA | DT_MOD_EXTTRIG);
                        } else {
                          dt2801_writecommand(
                                              DT_C_READ_AD | DT_MOD_DMA);
                        }
                        if (dt2801_checkforerror() != 0) {
                          printk("dt2801: blockstart-dma not ok\n");
                        }
                        adc->running = 1;
                    }
                }
            }
            break;
        case DT2801_IOCTL_ADC_BLOCKSTOP: {
                dprintk(( "dt2801_adc: block stop\n" ));
                if (dt2801->dmamode == 0) {
                    printk("dt2801_adc_ioctl: block only with dma\n");
                    return -1;
                } else {
                    dt2801_stop();
                    dt2801_clearerror();
                }
            }
            break;
        case DT2801_IOCTL_ADC_RESIDUE: {
            int stat, residue;

            if (dt2801->dmamode == 0) {
                    printk("dt2801_adc_ioctl: residue only with dma\n");
                    return -1;
                }
                stat = verify_area(VERIFY_WRITE, (char *)arg, sizeof(int));

                if ( stat != 0) { return stat; }

                residue = get_dma_residue(dt2801->dmachannel);

                put_fs_long(residue, (int*) arg);

            }
            break;
        case DT2801_IOCTL_ADC_POLARITY: {
            int stat;

                stat = verify_area(VERIFY_WRITE, (char *)arg, sizeof(int));
                if ( stat != 0) { return stat; }
                put_fs_long(adc->polarity, (int*) arg);
            }
            break;
        default:
             return dt2801_ioctl(inode, file, cmd, arg);

    }

    return 0;
    
}

static int dt2801_adc_read(struct inode *inode, struct file *file,
                       char* buffer, int count)
{
    Dt2801_ADC_t *adc = dt2801->adcs;

    int stat;
    int data;
    int d;


    dprintk(( "dt2801_adc:read minor = %d.\n",  MINOR(inode->i_rdev) ));

    switch (adc->mode)
    {
         case DT2801_ADC_IMMEDIATE:

            dprintk(( "dt2801_adc:read immediate.\n" ));

            if (count!=4 || adc->mode != DT2801_ADC_IMMEDIATE)
            {
                if (count!=4)
                return -EINVAL;
            }
            dt2801_writecommand(DT_C_READ_ADIM);
            dt2801_writedata(adc->gain);
            dt2801_writedata(adc->channel);
            dt2801_readdata2(&data);

            dprintk(( "dt2801_adc:read data = %d.\n", data ));

            if ((stat = verify_area(VERIFY_WRITE, buffer, count)) != 0)
                 return stat;
            memcpy_tofs(buffer, &data, count);
            return count;
        break;

         case DT2801_ADC_BLOCK: {
                 dprintk(( "dt2801_adc:read block.\n" ));
                 dprintk(( "dt2801_adc:read count = %d.\n", count ));
/*

                 printk( "dt2801_adc:read currnt = %d last = %d.\n",
                       adc->currnt, adc->last );
*/

             if (dt2801->dmamode == 0) {
                 printk("dt2801_adc: read block only with dma\n");
                 return -1;
             } else {
                 int residue;
                 if (dt2801->fake != 0) {
                     adc->currnt = fake_counter;
                 } else {
                     residue = get_dma_residue(dt2801->dmachannel);
                     /*                     printk("residue: %d\n", residue);  */

                     /* only transfer complete 2-byte values */
                     adc->currnt = adc->conv - ((residue+1)/sizeof(short));
                     /*                     printk("currnt: %d\n", adc->currnt);  */
                 }

                 if (adc->currnt > adc->conv)
                     return 0;
             
                 d = adc->currnt - adc->last;
                 if (d > count) {
                     d = count;
                 }
                 if (d < 0) {
                     return -(EINVAL);
                 }

                 if ((stat = verify_area(VERIFY_WRITE, buffer, d*sizeof(short))) != 0) {
                     return stat;
                 }

                 memcpy_tofs(buffer, adc->dmabuf+adc->last*sizeof(short), d*sizeof(short));

                 adc->last+=d;

                 return d;

             }
    
            }
        break;

         default:
            printk("dt2801_adc:bad mode %d.\n", adc->mode);
            return -1;
    }


    return 0;
}

static int adc_polarity = DT2801_ADC_UNIPOLAR;

static int dt2801_adc_reset(void)
{
    dt2801->adcs->channel  = 0;
    dt2801->adcs->startch = 0;
    dt2801->adcs->endch   = 0;
    dt2801->adcs->gain    = 0;
    dt2801->adcs->conv    = 0;
    dt2801->adcs->mode      = DT2801_ADC_IMMEDIATE;
    dt2801->adcs->last      = 0;
    dt2801->adcs->currnt      = 0;
    dt2801->adcs->running      = 0;
    dt2801->adcs->blocksize = 0;

    return 0;

}

int dt2801_adc_cleanup(void)
{

    if (dt2801->adcs->dmabuf != NULL) {
        kfree(dt2801->adcs->dmabuf);
    }

    if (dt2801->adcs != NULL) {
        kfree(dt2801->adcs);
    }

    return 0;
}

int dt2801_adc_init()
{
  /*
    dt2801->num_adcs = DT2801_NUM_ADCS;
    dt2801->adcs = (Dt2801_ADC_t *)
             kmalloc(sizeof( Dt2801_ADC_t) * (dt2801->num_adcs), GFP_KERNEL );
  */
    dt2801->adcs = (Dt2801_ADC_t*) kmalloc(sizeof( Dt2801_ADC_t), GFP_KERNEL );
    if ( dt2801->adcs == NULL) {
          return -(ENOMEM);
    }

    dt2801->adcs->inuse = 0;

    dt2801->adcs->polarity  = adc_polarity;

    dt2801_adc_reset();
    
    dt2801->adcs->dmabufsize = 0;
    dt2801->adcs->dmabuf      = NULL;

    printk("dt2801: adc_init: polarity(%d).\n", dt2801->adcs->polarity);
    return 0;
}

static int dt2801_adc_open(struct inode *inode, struct file *file)
{

    dprintk(( "dt2801_adc:open minor = %d.\n",  MINOR(inode->i_rdev) ));

    if (dt2801->inuse)
    {
        dprintk(( "dt2801_adc:already open %d.\n", dt2801->inuse ));
        return -(EBUSY);
    } else {
        dt2801->inuse = 1;

/*
        dt2801_stop();
        dt2801_reset();
*/

        dt2801_adc_reset();
    }

    return 0;
}

static void dt2801_adc_release(struct inode *inode, struct file *file)
{
  /*    Dt2801_ADC_t *adc = dt2801->adcs; */

    dprintk(( "dt2801_adc:release minor = %d.\n",  MINOR(inode->i_rdev) ));

    dt2801_release(inode, file);
}

struct file_operations dt2801_adc_fops = {
    NULL,                /* seek */
    dt2801_adc_read,     /* read */
    NULL,                /* write */
    NULL,                /* readdir */
    NULL,                /* select */
    dt2801_adc_ioctl,    /* ioctl */
    NULL,                /* mmap */
    dt2801_adc_open,     /* open */
    dt2801_adc_release   /* release */
};
