/*
 * Device Driver for DataTranslation DT2801
 *
 * $Id: dt2801_dac.c,v 1.2 1996/03/13 09:37:44 martin Exp martin $
 * $Source: /home/martin/Software/DAQ/drivers/dt2801/RCS/dt2801_dac.c,v $
 *
 *
 * $Log: dt2801_dac.c,v $
 * Revision 1.2  1996/03/13 09:37:44  martin
 * Disabeled for the moment
 *
 * Revision 1.1  1996/03/04 07:20:49  martin
 * Initial revision
 *
 *
 */

#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/tty.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/major.h>
#include <linux/malloc.h>
#include <linux/timer.h>

#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/dma.h>

#ifdef MODULE
#include <linux/version.h>
#else
#error "dt2801_dac must be compiled as a module"
#endif

#include "dt2801.h"
#include "dt2801_dac.h"

static Dt2801_DAC_t dacs[DT2801_NUM_DACS];

static int dt2801_dac_write(struct inode *inode, struct file *file,
                       const char* buffer, int count)
{
    int minor = MINOR(inode->i_rdev);
    Dt2801_DAC_t* dac = &dacs[1];

    int nr = minor & 0x3;

    dprintk(( "dt2801_dac_write: minor = %d  nr = %d.\n", minor, nr ));

    switch (dac->mode)
    {
        case DT2801_DAC_IMMEDIATE: {
            int data;

            dprintk(( "dt2801_dac_write: IMMEDIATE.\n" ));

            if (count != 4)
            {
                return -(EINVAL);
            }

            data = get_fs_long(buffer);
            dt2801_writecommand(DT_C_WRITE_DAIM);
            dt2801_writedata(nr);
            dt2801_writedata2(data);

            return count;
      
        } break;

        case DT2801_DAC_BLOCK: {
        }

    }

    return 0;
}

static int dt2801_dac_ioctl(struct inode *inode, struct file *file,
                       unsigned int cmd, unsigned long arg)
{
    int minor = MINOR(inode->i_rdev);
    Dt2801_DAC_t* dac = &dacs[1];

    int nr = minor & 0x3;

    dprintk(( "dt2801_dac_ioctl: minor = %d  cmd = %u.\n", minor, cmd ));

    switch( cmd)
    {
        case DT2801_IOCTL_DAC_SET_PARAMS: {
            Dt2801_DACP_t dacp;

            memcpy_fromfs(&dacp, (char*) arg, sizeof(dacp));
            dprintk(( "dt2801_dac_ioctl: conv = %d\n.", dacp.conv ));

            dt2801_writecommand(DT_C_SET_DA);

            dt2801_writedata(nr);
            dt2801_writedata2(dacp.conv);

            dac->conv = dacp.conv;
            return dt2801_checkforerror();
        }
        break;

        case DT2801_IOCTL_DAC_SET_MODE: {
            dprintk(( "dt2801_dac: ioctl setdacmode.\n" ));
            switch (arg)
            {
            case DT2801_DAC_IMMEDIATE:
                dac->mode = arg;
            break;
/*
            case DT2801_DAC_BLOCK:
                dac->mode = arg;
            break;
*/
            default:
                dprintk(( "dt2801_dac_ioctl: wrong arg %lu to setmode.\n", arg ));
                return -(EINVAL);
            }
        }
        break;

        default:
            return dt2801_ioctl(inode, file, cmd, arg);
    }

    return 0;
}

int dt2801_dac_reset(void)
{

    return 0;
}

int dt2801_dac_cleanup(void)
{
    return 0;
}

int dt2801_dac_init()
{

    int data;
    int nr;
/*
    if ( (dt2801->dacs = (Dt2801_DAC_t *)
             kmalloc( sizeof( Dt2801_DAC_t) * (DT2801_NUM_DACS+1), GFP_KERNEL )) == NULL)
          return -(ENOMEM);
*/

    /* Reset to 0 Volts */
    data = 2048; nr = 0;
    dt2801_writecommand(DT_C_WRITE_DAIM);
    dt2801_writedata(nr);
    dt2801_writedata2(data);
    data = 2048; nr = 1;
    dt2801_writecommand(DT_C_WRITE_DAIM);
    dt2801_writedata(nr);
    dt2801_writedata2(data);

    return 0;
}

static int dt2801_dac_open(struct inode *inode, struct file *file)
{

    dprintk(( "dt2801_dac_open: minor = %d.\n", MINOR(inode->i_rdev) ));

    if (dt2801->inuse)
    {
        return -(EBUSY);
    } else {
/*
        dt2801_stop();
        dt2801_reset();
*/
        dt2801->inuse = 1;
    }

    return 0;
}

static void dt2801_dac_release(struct inode *inode, struct file *file)
{
    DT_PRINTK("dt2801_dac: release\n");

    dt2801_release(inode, file);
}

struct file_operations dt2801_dac_fops = {
    NULL,                /* seek */
    NULL,                /* read */
    dt2801_dac_write,    /* write */
    NULL,                /* readdir */
    NULL,                /* select */
    dt2801_dac_ioctl,    /* ioctl */
    NULL,                /* mmap */
    dt2801_dac_open,     /* open */
    dt2801_dac_release   /* release */
};
